@@ -1,6 +1,6 @@
 package Git::Raw::Diff;
 {
-  $Git::Raw::Diff::VERSION = '0.26'; # TRIAL
+  $Git::Raw::Diff::VERSION = '0.30'; # TRIAL
 }
 
 use strict;
@@ -14,7 +14,7 @@ Git::Raw::Diff - Git diff class
 
 =head1 VERSION
 
-version 0.26
+version 0.30
 
 =head1 DESCRIPTION
 
@@ -29,7 +29,7 @@ B<WARNING>: The API of this module is unstable and may change without warning
 
 Merge the given diff with the C<Git::Raw::Diff> C<$from>.
 
-=head2 patch( $callback )
+=head2 print( $format, $callback )
 
 Generate text output from the diff object. The C<$callback> will be called for
 each line of the diff with two arguments: the first one represents the type of
@@ -38,11 +38,31 @@ for deletions, C<"file"> for file headers, C<"hunk"> for hunk headers or
 C<"bin"> for binary data) and the second argument contains the content of the
 patch line.
 
-=head2 compact( $callback )
+The C<$format> can be one of the following:
 
-Generate compact text output from a diff object. Differently from C<patch()>,
-this function only passes the names and statuses of changed files to the
-callback.
+=over 4
+
+=item * "patch"
+
+Full git diff.
+
+=item * "patch_header"
+
+Only the file headers of the diff.
+
+=item * "raw"
+
+Like C<git diff --raw>.
+
+=item * "name_only"
+
+Like C<git diff --name-only>.
+
+=item * "name_status"
+
+Like C<git diff --name-status>.
+
+=back
 
 =head1 AUTHOR
 
