@@ -1,6 +1,6 @@
 package Git::Raw::Push;
 {
-  $Git::Raw::Push::VERSION = '0.26'; # TRIAL
+  $Git::Raw::Push::VERSION = '0.30'; # TRIAL
 }
 
 use strict;
@@ -12,7 +12,7 @@ Git::Raw::Push - Git push class
 
 =head1 VERSION
 
-version 0.26
+version 0.30
 
 =head1 SYNOPSIS
 
@@ -26,7 +26,7 @@ version 0.26
 
     # set the acquire credentials callback
     $remote -> callbacks({
-      credentials => sub { Git::Raw::Cred -> plaintext($usr, $pwd) }
+      credentials => sub { Git::Raw::Cred -> userpass($usr, $pwd) }
     });
 
     # connect the remote
