@@ -1,6 +1,6 @@
 package Git::Raw::Reference;
 {
-  $Git::Raw::Reference::VERSION = '0.26'; # TRIAL
+  $Git::Raw::Reference::VERSION = '0.30'; # TRIAL
 }
 
 use strict;
@@ -14,7 +14,7 @@ Git::Raw::Reference - Git reference class
 
 =head1 VERSION
 
-version 0.26
+version 0.30
 
 =head1 DESCRIPTION
 
@@ -29,6 +29,14 @@ B<WARNING>: The API of this module is unstable and may change without warning
 
 Retrieve the reference with name C<$name> in C<$repo>.
 
+=head2 create( $name, $repo, $object [, $force] )
+
+Creates and returns a new direct reference named C<$name> in C<$repo> pointing
+to C<$object>. C<$object> can be a L<Git::Raw::Blob>, L<Git::Raw::Commit>,
+or a L<Git::Raw::Tree> object.  If C<$force> is a truthy value, any existing
+reference is overwritten.  If C<$force> is falsy (the default) and a reference
+named C<$name> already exists, an error is thrown.
+
 =head2 delete( )
 
 Delete the reference. The L<Git::Raw::Reference> object must not be accessed
@@ -42,11 +50,17 @@ Retrieve the name of the reference.
 
 Retrieve the type of the reference. Can be either C<"direct"> or C<"symbolic">.
 
-=head2 target( )
+=head2 target( [$new_target] )
 
-Retrieve the target of the reference. This function returns either an object
-(L<Git::Raw::Blob>, L<Git::Raw::Commit>, L<Git::Raw::Tag> or L<Git::Raw::Tree>)
-for direct references, or another reference for symbolic references.
+Retrieve the target of the reference. If the C<$new_target> parameter of type
+L<Git::Raw::Reference> is passed, the reference will be changed to point to it.
+
+Note that updating the target will invalidate all existing handles to the
+reference.
+
+This function returns either an object (L<Git::Raw::Blob>, L<Git::Raw::Commit>,
+L<Git::Raw::Tag> or L<Git::Raw::Tree>) for direct references, or another
+reference for symbolic references.
 
 =head2 owner( )
 
