@@ -1,6 +1,6 @@
 package Git::Raw::Remote;
 {
-  $Git::Raw::Remote::VERSION = '0.26'; # TRIAL
+  $Git::Raw::Remote::VERSION = '0.30'; # TRIAL
 }
 
 use strict;
@@ -14,7 +14,7 @@ Git::Raw::Remote - Git remote class
 
 =head1 VERSION
 
-version 0.26
+version 0.30
 
 =head1 SYNOPSIS
 
@@ -28,7 +28,11 @@ version 0.26
 
     # set the acquire credentials callback
     $remote -> callbacks({
-      credentials => sub { Git::Raw::Cred -> plaintext($usr, $pwd) }
+      'credentials' => sub { Git::Raw::Cred -> userpass($usr, $pwd) }
+      'update_tips' => sub {
+        my ($ref, $a, $b) = @_);
+        print "Updated $ref: $a -> $b", "\n";
+      }
     });
 
     # connect the remote
@@ -41,6 +45,10 @@ version 0.26
     # disconnect
     $remote -> disconnect;
 
+    my $empty_repo = Git::Raw::Repository -> new;
+    my $inmem_remote = Git::Raw::Remote -> create_inmemory($repo, undef, $url);
+    my $list = $inmem_remote -> ls;
+
 =head1 DESCRIPTION
 
 A C<Git::Raw::Remote> represents a Git remote.
@@ -55,6 +63,10 @@ B<WARNING>: The API of this module is unstable and may change without warning
 Create a remote with the default fetch refspec and add it to the repository's
 configuration.
 
+=head2 create_inmemory( $repo, $fetch_refspec, $url )
+
+Create a remote in memory.
+
 =head2 load( $repo, $name )
 
 Load an existing remote.
@@ -77,6 +89,28 @@ Add a fetch spec to the remote.
 
 Add a push spec to the remote.
 
+=head2 ls( )
+
+Retrieve the list of refs at the remote. Returns a hash reference containing
+C<key => value> pairs where the key is the name of the reference, and the value
+is a hash reference containing the following values:
+
+=over 4
+
+=item * "local"
+
+Whether the reference exists locally.
+
+=item * "id"
+
+The object ID of the reference.
+
+=item * "lid"
+
+The local object ID of the reference (optional).
+
+=back
+
 =head2 callbacks( \%callbacks )
 
 =over 4
@@ -87,25 +121,32 @@ The callback to be called any time authentication is required to connect to the
 remote repository. The callback receives a string containing the URL of the
 remote, and it must return a L<Git::Raw::Cred> object.
 
+=item * "progress"
+
+Textual progress from the remote. Text send over the progress side-band will be
+passed to this function (this is the 'counting objects' output). The callback
+receives a string containing progress information.
+
+=item * "completion"
+
+Completion is called when different parts of the download process are done
+(currently unused).
+
+=item * "transfer_progress"
+
+During the download of new data, this will be regularly called with the current
+count of progress done by the indexer. The callback receives the following integers:
+C<total_objects>, C<received_objects>, C<local_objects>, C<total_deltas>,
+C<indexed_deltas> and C<received_bytes>.
+
+=item * "update_tips"
+
+Each time a reference is updated locally, this function will be called with
+information about it. The callback receives a string containing the name of the
+reference that was updated, and the two OID's C<a> before and after C<b> the update.
+
 =back
 
-=head2 cred_acquire( $callback )
-
-Run $callback any time authentication is required to connect to the remote
-repository. The callback receives a string containing the URL of the remote, and
-it must return a L<Git::Raw::Cred> object.
-
-Note that this method is deprecated in favor of C<callbacks()>
-
-=cut
-
-sub cred_acquire {
-	my ($self, $cb) = @_;
-
-	my %callbacks = { 'credentials' => $cb };
-	$self -> callbacks(\%callbacks);
-}
-
 =head2 connect( $direction )
 
 Connect to the remote. The direction can be either C<"fetch"> or C<"push">.
